#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
Userel

A extend ForeignKey field for user relation.

AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.importlib import import_module

import backends

settings.USEREL_BACKEND = getattr(settings, 'USEREL_BACKEND', backends.UserelDefaultBackend)

def load_backend(path):
    """load userel backend from string path"""
    i = path.rfind('.')
    module ,attr = path[:i], path[i+1:]
    try:
        mod = import_module(module)
    except ImportError, e:
        raise ImproperlyConfigured('Error importing userel backend %s: "%s"' % (path, e))
    except ValueError, e:
        raise ImproperlyConfigured('Error importing userel backend. Is USEREL_BACKEND a correctly defined?')
    try:
        cls = getattr(mod, attr)
    except AttributeError:
        raise ImproperlyConfigured('Module "%s" does not define a "%s" userel backend' % (module, attr))
    
    if not hasattr(cls, 'get_user'):
        raise ImproperlyConfigured('Error userel backend must have "get_user" method Please define it in %s.' % cls)
    return cls()

def get_backend_class():
    """get userel backend"""
    backend = settings.USEREL_BACKEND
    if isinstance(backend, basestring):
        return load_backend(backend)
    elif isinstance(backend, object) and hasattr(backend, 'get_user'):
        return backend
    else:
        raise ImproperlyConfigured('Error userel backend must have "get_user" method Please define it in %s.' % backend)

def get_backend():
    backend_class = get_backend_class()
    return backend_class()



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
Mini blog views


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.views.generic import ListView
from django.views.generic import DetailView
from django.views.generic import CreateView
from django.views.generic import UpdateView
from django.views.generic import DeleteView
from django.core.urlresolvers import reverse

from models import Entry
from forms import EntryForm

class EntryListView(ListView):
    model = Entry
class EntryDetailView(DetailView):
    model = Entry
    slug_field = 'title'

class EntryCreateView(CreateView):
    form_class = EntryForm
    model = Entry
class EntryUpdateView(UpdateView):
    form_class = EntryForm
    model = Entry
class EntryDeleteView(DeleteView):
    model = Entry
    def get_success_url(self):
        return reverse('blog-entry-list')





from django.conf.urls.defaults import patterns, include, url

# Uncomment the next two lines to enable the admin:
from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    url(r'^admin/', include(admin.site.urls)),
    url(r'^registration/login/$', 'django.contrib.auth.views.login',
        name='login'),
    url(r'^registration/logout/$', 'django.contrib.auth.views.logout',
        name='logout'),
    url(r'^', include('miniblog.blog.urls')),
)



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
backends for django-userel

AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from middlewares import get_request

class UserelDefaultBackend(object):
    """Userel default backend
    
    Get current user from request stored in thread_locals.
    Return None when request.user is not detected include request.user
    is AnonymousUser

    """

    def __init__(self):
        required_middleware = 'userel.middlewares.UserelDefaultBackendMiddleware'
        if required_middleware not in settings.MIDDLEWARE_CLASSES:
            raise ImproperlyConfigured(
                    'Error "%s" is not found in MIDDLEWARE_CLASSES. '
                    'It is required to use UserelDefaultBackend' % required_middleware)
    def _get_user_model(self):
        """get user model class"""
        from django.contrib.auth.models import User
        return User

    def _get_request(self):
        """get current request"""
        return get_request()

    def get_user(self):
        """get current user"""
        request = self._get_request()
        if request and getattr(request, 'user', None):
            if isinstance(request.user, self._get_user_model()):
                return request.user
        # AnonymousUser
        return None

class UserelSystemUserBackend(UserelDefaultBackend):
    """Userel System user backend
    
    Get current user from request stored in thread_locals.
    Return System user when request.user is not detected include request.user
    is AnonymousUser

    System user is detected with ``get_system_user``

    """
    """Author default backend"""

    def _get_filter_kwargs(self):
        """get kwargs for filtering user"""
        return {'pk': 1}

    def get_system_user(self):
        """get system user"""
        user_model = self._get_user_model()
        user = user_model._default_manager.get(**self._get_filter_kwargs())
        return user

    def get_user(self):
        """get current user"""
        return super(UserelSystemUserBackend, self).get_user() or self.get_system_user()



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
short module explanation


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.conf import settings
from django.db import models
from django.contrib.auth import models as auth_models
from django.contrib.auth.management import create_superuser
from django.db.models import signals

settings.AUTO_CREATE_USER = getattr(settings, 'AUTO_CREATE_USER', True)

if settings.DEBUG and settings.AUTO_CREATE_USER:
    # From http://stackoverflow.com/questions/1466827/ --
    #
    # Prevent interactive question about wanting a superuser created. (This code
    # has to go in this otherwise empty "models" module so that it gets processed by
    # the "syncdb" command during database creation.)
    #
    # Create our own test user automatically.
    def create_testuser(app, created_models, verbosity, **kwargs):
        USERNAME = getattr(settings, 'QWERT_AUTO_CREATE_USERNAME', 'admin')
        PASSWORD = getattr(settings, 'QWERT_AUTO_CREATE_PASSWORD', 'password')
        EMAIL = getattr(settings, 'QWERT_AUTO_CREATE_EMAIL', 'x@x.com')

        if getattr(settings, 'QWERT_AUTO_CREATE_USER', None):
            User = models.get_model(*settings.QWERT_AUTO_CREATE_USER.rsplit('.', 1))
        else:
            from django.contrib.auth.models import User

        try:
            User.objects.get(username=USERNAME)
        except User.DoesNotExist:
            print '*' * 80
            print 'Creating test user -- login: %s, password: %s' % (USERNAME, PASSWORD)
            print '*' * 80
            assert User.objects.create_superuser(USERNAME, EMAIL, PASSWORD)
        else:
            print 'Test user already exists. -- login: %s, password: %s' % (USERNAME, PASSWORD)
    signals.post_syncdb.disconnect(
        create_superuser,
        sender=auth_models,
        dispatch_uid='django.contrib.auth.management.create_superuser')
    signals.post_syncdb.connect(create_testuser,
        sender=auth_models, dispatch_uid='common.models.create_testuser')



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
middlewares for django-userel


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local

__all__ = ['get_request', 'UserelDefaultBackendMiddleware']
_thread_locals = local()

def get_request():
    """Get request stored in current thread"""
    return getattr(_thread_locals, 'request', None)

class UserelDefaultBackendMiddleware(object):
    """Middleware to store request instance in threadlocals"""
    def process_request(self, request):
        _thread_locals.request = request



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
Unittest module of ...


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.test import TestCase
from django.db import IntegrityError


class EntryViewTestCase(TestCase):
    fixtures = ['test.yaml']

    def test_list(self):
        response = self.client.get('/')
        self.assertEqual(response.status_code, 200)

    def test_detail(self):
        response = self.client.get('/foo/')
        self.assertEqual(response.status_code, 200)

    def test_create(self):
        response = self.client.get('/create/')
        self.assertEqual(response.status_code, 200)

    def test_update(self):
        response = self.client.get('/update/1/')
        self.assertEqual(response.status_code, 200)

    def test_delete(self):
        response = self.client.get('/delete/1/')
        self.assertEqual(response.status_code, 200)

    def test_userel_fields(self):
        from django.contrib.auth.models import User
        from ..models import Entry
        self.client.logout()
        # Because with Non logged in user, created_by, updated_by would be
        # None with the UserelDefaultBackend
        # and that is not allowed
        self.assertRaises(IntegrityError, self.client.post, '/create/', {
                'title': 'barbar',
                'body': 'barbar'
            })

        assert self.client.login(username='admin', password='password')
        response = self.client.post('/create/', {
                'title': 'barbar',
                'body': 'barbar'
            })
        # if post success, redirect occur
        self.assertEqual(response.status_code, 302)

        admin = User.objects.get(username='admin')
        entry = Entry.objects.get(title='barbar')
        self.assertEqual(entry.created_by, admin)
        self.assertEqual(entry.updated_by, admin)

        self.client.logout()
        assert self.client.login(username='foo', password='password')
        response = self.client.post('/update/%d/' % entry.pk, {
                'title': 'barbarbar',
                'body': 'barbarbar'
            })
        # if post success, redirect occur
        self.assertEqual(response.status_code, 302)

        foo = User.objects.get(username='foo')
        entry = Entry.objects.get(pk=entry.pk)
        self.assertEqual(entry.created_by, admin)
        self.assertEqual(entry.updated_by, foo)
